@@ -88,8 +88,14 @@ def _save(state: _State, t: Scalar) -> _State:
     )
 
 
-def _clip_to_end(tnext, t1):
-    return jnp.where(tnext > t1 - 1e-6, t1, tnext)
+def _clip_to_end(tprev, tnext, t1, keep_step):
+    if tnext.dtype is jnp.dtype("float64"):
+        tol = 1e-10
+    else:
+        tol = 1e-6
+    clip = tnext > t1 - tol
+    tclip = jnp.where(keep_step, t1, tprev + 0.5 * (t1 - tprev))
+    return jnp.where(clip, tclip, tnext)
 
 
 def loop(
@@ -161,8 +167,8 @@ def loop(
         # The 1e-6 tolerance means that we don't end up with too-small intervals for
         # dense output, which then gives numerically unstable answers due to floating
         # point errors.
-        tnext = _clip_to_end(tnext, t1)
         tprev = jnp.minimum(tprev, t1)
+        tnext = _clip_to_end(tprev, tnext, t1, keep_step)
 
         # The other parts of the mutable state are kept/not-kept (based on whether the
         # step was accepted) by the stepsize controller. But it doesn't get access to
@@ -403,7 +409,7 @@ def loop(
 
                     def _body_fun(_state):
                         _step, _t = _state
-                        return _step + 1, _clip_to_end(_t + dt0, t1)
+                        return _step + 1, _clip_to_end(_t, _t + dt0, t1, True)
 
                     compiled_num_steps, _ = lax.while_loop(
                         _cond_fun, _body_fun, (0, t0)
@@ -600,12 +606,12 @@ def diffeqsolve(
     # Error checking
     if dt0 is not None:
         msg = (
-            "Must have (t1 - t0) * dt0 > 0, we instead got: "
-            f"t1, with value: {t1} and type {type(t1)}, "
-            f"t0, with value: {t0} and type {type(t0)} and, "
-            f"dt0, with value: {dt0} and type {type(dt0)}"
+            "Must have (t1 - t0) * dt0 >= 0, we instead got "
+            f"t1 with value {t1} and type {type(t1)}, "
+            f"t0 with value {t0} and type {type(t0)}, "
+            f"dt0 with value {dt0} and type {type(dt0)}"
         )
-        error_if(lambda: (t1 - t0) * dt0 <= 0, msg)
+        error_if((t1 - t0) * dt0 < 0, msg)
 
     # Error checking
     term_leaves, term_structure = jax.tree_flatten(
@@ -748,10 +754,10 @@ def diffeqsolve(
         # We have no way of knowing how many steps we'll actually end up taking, and
         # XLA doesn't support dynamic shapes. So we just have to allocate the maximum
         # amount of steps we can possibly take.
-        error_if(
-            max_steps is None,
-            "`max_steps=None` is incompatible with `saveat.steps=True`",
-        )
+        if max_steps is None:
+            raise ValueError(
+                "`max_steps=None` is incompatible with `saveat.steps=True`"
+            )
         out_size += max_steps
     if saveat.t1 and not saveat.steps:
         out_size += 1
@@ -766,10 +772,10 @@ def diffeqsolve(
     result = jnp.array(RESULTS.successful)
     if saveat.dense:
         error_if(t0 == t1, "Cannot save dense output if t0 == t1")
-        error_if(
-            max_steps is None,
-            "`max_steps=None` is incompatible with `saveat.dense=True`",
-        )
+        if max_steps is None:
+            raise ValueError(
+                "`max_steps=None` is incompatible with `saveat.dense=True`"
+            )
         (
             _,
             _,
